VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "CRat"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = False
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' *****************************************************************
'  Copyright 1994-2000, Zane Thomas, http://www.mabry.com
'  Modified by Karl E. Peterson, May 2000, for publication
'  in VBPJ and at http://www.mvps.org/vb/
' *****************************************************************
'  Authors grant royalty-free rights to use this code within
'  compiled applications. Selling or otherwise distributing
'  this source code is not allowed without author's express
'  permission.
' ******************************************************************
'  http://www.dictionary.com/cgi-bin/dict.pl?term=rational%20number
' ******************************************************************
Option Explicit

Private m_Numer As Long
Private m_Denom As Long
Private m_AutoReduce As Boolean

' ************************************************
'  Initialize
' ************************************************
Private Sub Class_Initialize()
   m_Numer = 0
   m_Denom = 1
   m_AutoReduce = True
End Sub

' ************************************************
'  Public Properties
' ************************************************
Public Property Let AutoReduce(ByVal NewVal As Boolean)
   m_AutoReduce = NewVal
End Property

Public Property Get AutoReduce() As Boolean
   AutoReduce = m_AutoReduce
End Property

Public Property Let Denominator(ByVal NewVal As Long)
   If NewVal = 0 Then
      ' Denominator cannot be zero
      Err.Raise Number:=11, Source:="CRat.Denominator", _
         Description:="Division by zero."
   Else
      m_Denom = NewVal
   End If
End Property

Public Property Get Denominator() As Long
   Denominator = m_Denom
End Property

Public Property Let Numerator(ByVal NewVal As Long)
   m_Numer = NewVal
End Property

Public Property Get Numerator() As Long
   Numerator = m_Numer
End Property

' ************************************************
'  Public Methods - Math
' ************************************************
Public Sub Reduce()
   Dim Denom As Long
   ' Calc GCD and divide both to reduce.
   Denom = Gcd(m_Numer, m_Denom)
   m_Numer = m_Numer / Denom
   m_Denom = m_Denom / Denom
End Sub

Public Function Add(aRat As CRat) As CRat
   Dim result As New CRat
   '
   ' Add a rat, return result as a new rat.
   '
   ' n1   n2   n1d2 + n2d1
   ' -- + -- = -----------
   ' d1   d2       d1d2
   '
   result.Numerator = m_Numer * aRat.Denominator + aRat.Numerator * m_Denom
   result.Denominator = m_Denom * aRat.Denominator
   If m_AutoReduce Then result.Reduce
   Set Add = result
End Function

Public Function Subtract(aRat As CRat) As CRat
   Dim result As New CRat
   '
   ' Subtract a rat, return result as a new rat
   '
   ' n1   n2   n1d2 - n2d1
   ' -- - -- = -----------
   ' d1   d2       d1d2
   '
   result.Numerator = m_Numer * aRat.Denominator - aRat.Numerator * m_Denom
   result.Denominator = m_Denom * aRat.Denominator
   If m_AutoReduce Then result.Reduce
   Set Subtract = result
End Function

Public Function Multiply(aRat As CRat) As CRat
   Dim result As New CRat
   '
   ' Multiply a rat, return result as a new rat
   '
   ' n1   n2   n1d2
   ' -- * -- = ----
   ' d1   d2   d1d2
   '
   result.Numerator = m_Numer * aRat.Numerator
   result.Denominator = m_Denom * aRat.Denominator
   If m_AutoReduce Then result.Reduce
   Set Multiply = result
End Function

Public Function Divide(aRat As CRat) As CRat
   Dim result As New CRat
   '
   ' Divide by a rat, return result as a new rat
   '
   ' n1/d1   n1d2
   ' ----- = ----
   ' n2/d2   d1n2
   '
   result.Numerator = m_Numer * aRat.Denominator
   result.Denominator = m_Denom * aRat.Numerator
   If m_AutoReduce Then result.Reduce
   Set Divide = result
End Function

' ************************************************
'  Public Methods - Utility/Display
' ************************************************
Public Function AsFloat() As Double
   AsFloat = m_Numer / m_Denom
End Function

Public Function AsString(Optional ByVal ShowWhole As Boolean = False) As String
   If m_AutoReduce Then Me.Reduce
   
   If m_Numer Mod m_Denom = 0 Then
      ' Output should be like "2"
      AsString = m_Numer \ m_Denom
   
   ElseIf Abs(m_Numer) > Abs(m_Denom) And ShowWhole = True Then
      ' Output should be like "1-3/8"
      AsString = CStr(m_Numer \ m_Denom) & "-" & _
         CStr(Abs(m_Numer Mod m_Denom)) & "/" & _
         CStr(Abs(m_Denom))
   
   Else
      ' Output should be like "11/8"
      AsString = m_Numer & "/" & m_Denom
   End If
End Function

Public Function MakeRat(ByVal Numer As Long, ByVal Denom As Long) As CRat
   Dim result As New CRat
   ' Provide a rat for on-the-fly calcs
   result.SetValues Numer, Denom
   Set MakeRat = result
End Function

Public Sub SetValue(ByVal NewVal As Double, Optional ByVal HighDenom As Long = 64)
   Dim Numer As Long
   Dim Denom As Long
   Dim Whole As Long
   ' Create a rat from a single floating-point value,
   ' and the largest value acceptable for denom.
   Whole = Abs(Fix(NewVal))
   Denom = Abs(HighDenom)
   Numer = Denom * (Abs(NewVal) - Whole)
   If Numer Then
      m_Numer = ((Whole * Denom) + Numer) * Sgn(NewVal)
   Else
      m_Numer = (Whole * Denom) * Sgn(NewVal)
   End If
   Me.Denominator = Denom * Sgn(HighDenom)
End Sub

Public Sub SetValues(ByVal Numer As Long, ByVal Denom As Long)
   ' Create a rat from passed numer and denom.
   m_Numer = Numer
   Me.Denominator = Denom
End Sub

' ************************************************
'  Private Methods
' ************************************************
Private Function Gcd(ByVal a As Long, ByVal b As Long) As Long
   ' Find greatest common denominator.
   If (b = 0) Then
      Gcd = a
   Else
      Gcd = Gcd(b, a Mod b)
   End If
End Function


